@@ -12,32 +12,11 @@
  *
  * --------------------------------------------------------------------------
  *
- * Copyright (c) 2009-2010, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2009-Present, Redis Ltd.
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *   * Redistributions of source code must retain the above copyright notice,
- *     this list of conditions and the following disclaimer.
- *   * Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *   * Neither the name of Redis nor the names of its contributors may be used
- *     to endorse or promote products derived from this software without
- *     specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * Licensed under your choice of the Redis Source Available License 2.0
+ * (RSALv2) or the Server Side Public License v1 (SSPLv1).
  */
 
 /* Memory layout of a zipmap, for the map "foo" => "bar", "hello" => "world":
@@ -399,7 +378,7 @@ int zipmapValidateIntegrity(unsigned char *zm, size_t size, int deep) {
     while(*p != ZIPMAP_END) {
         /* read the field name length encoding type */
         s = zipmapGetEncodedLengthSize(p);
-        /* make sure the entry length doesn't rech outside the edge of the zipmap */
+        /* make sure the entry length doesn't reach outside the edge of the zipmap */
         if (OUT_OF_RANGE(p+s))
             return 0;
 
@@ -408,13 +387,13 @@ int zipmapValidateIntegrity(unsigned char *zm, size_t size, int deep) {
         p += s; /* skip the encoded field size */
         p += l; /* skip the field */
 
-        /* make sure the entry doesn't rech outside the edge of the zipmap */
+        /* make sure the entry doesn't reach outside the edge of the zipmap */
         if (OUT_OF_RANGE(p))
             return 0;
 
         /* read the value length encoding type */
         s = zipmapGetEncodedLengthSize(p);
-        /* make sure the entry length doesn't rech outside the edge of the zipmap */
+        /* make sure the entry length doesn't reach outside the edge of the zipmap */
         if (OUT_OF_RANGE(p+s))
             return 0;
 
@@ -425,7 +404,7 @@ int zipmapValidateIntegrity(unsigned char *zm, size_t size, int deep) {
         p += l+e; /* skip the value and free space */
         count++;
 
-        /* make sure the entry doesn't rech outside the edge of the zipmap */
+        /* make sure the entry doesn't reach outside the edge of the zipmap */
         if (OUT_OF_RANGE(p))
             return 0;
     }
@@ -476,12 +455,12 @@ static void zipmapRepr(unsigned char *p) {
 }
 
 #define UNUSED(x) (void)(x)
-int zipmapTest(int argc, char *argv[], int accurate) {
+int zipmapTest(int argc, char *argv[], int flags) {
     unsigned char *zm;
 
     UNUSED(argc);
     UNUSED(argv);
-    UNUSED(accurate);
+    UNUSED(flags);
 
     zm = zipmapNew();
 
